module Main exposing (Model, Msg(..), arg, init, main, update, view)

import Browser
import Html exposing (Html, button, div, pre, text, textarea)
import Html.Attributes exposing (attribute, class, contenteditable, id, style)
import Html.Events exposing (on, onClick)
import Json.Decode exposing (string)
import SvgBob



{--code which detects lines and connections
also algorithmn to connect these lines
--}


type alias Model =
    { grid : SvgBob.Model
    }


type Msg
    = Input String
    | Convert


init : Model
init =
    { grid = SvgBob.init arg }


view : Model -> Html Msg
view model =
    div [ style "display" "flex" ]
        [ div []
            [ button
                [ onClick Convert
                ]
                [ text "Convert >>" ]
            ]
        , SvgBob.getSvg [ style "width" "100%", attribute "vector-effect" "non-scaling-stroke" ] model.grid
        ]


update : Msg -> Model -> Model
update msg model =
    case msg of
        Convert ->
            model

        Input asciiText ->
            { model | grid = SvgBob.init asciiText }


main : Program () Model Msg
main =
    Browser.sandbox
        { init = init
        , update = update
        , view = view
        }


arg : String
arg =
    """

Resistor
     .--.......--.
  .-(   |||||||   )-.
 /   '--'''''''--'   \\


+------+   +-----+   +-----+   +-----+
|      |   |     |   |     |   |     |
| Foo  +-->| Bar +---+ Baz |<--+ Moo |
|      |   |     |   |     |   |     |
+------+   +-----+   +--+--+   +-----+
              ^         |
              |         V
.-------------+-----------------------.
| Hello here and there and everywhere |
'-------------------------------------'


                        ____________
   .--------------.     \\           \\
  / a == b         \\     \\           \\     __________
 (    &&            )     ) process   )    \\         \\
  \\ 'string' ne '' /     /           /     / process /
   '--------------'     /___________/     /_________/

  User code  ^               ^ OS code
              \\             /
               \\        .--'
                \\      /
  User code  <--- Mode ----> OS code
                /      \\
            .--'        \\___
           /                \\
          v                  v
       User code            OS code

             .---.  .---. .---.  .---.    .---.  .---.
    OS API   '---'  '---' '---'  '---'    '---'  '---'
               |      |     |      |        |      |
               v      v     |      v        |      v
             .------------. | .-----------. |  .-----.
             | Filesystem | | | Scheduler | |  | MMU |
             '------------' | '-----------' |  '-----'
                    |       |      |        |
                    v       |      |        v
                 .----.     |      |    .---------.
                 | IO |<----'      |    | Network |
                 '----'            |    '---------'
                    |              |         |
                    v              v         v
             .---------------------------------------.
             |                  HAL                  |
             '---------------------------------------'


   ____[]
  | ___ |
  ||   ||  device
  ||___||  loads
  | ooo |----------------------------------------------------------.
  | ooo |    |                          |                          |
  | ooo |    |                          |                          |
  '_____'    |                          |                          |
             |                          |                          |
             v                          v                          v
   .-------------------.  .---------------------------.  .-------------------.
   | Loadable module C |  |     Loadable module A     |  | Loadable module B |
   '-------------------'  |---------------------------|  |   (instrumented)  |
             |            |         .-----.           |  '-------------------'
             '------------+-------->| A.o |           |             |
                 calls    |         '-----'           |             |
                          |    .------------------.   |             |
                          |   / A.instrumented.o /<---+-------------'
                          |  '------------------'     |    calls
                          '---------------------------'


                                        .--> Base::Class::Derived_A
                                       /
                                      .----> Base::Class::Derived_B
      Something -------.             /         \\
                        \\           /           +---> Base::Class::Derived
      Something::else    \\         /             \\
            \\             \\       /               '--> Base::Class::Derived
             \\             \\     /
              \\             \\   .-----------> Base::Class::Derived_C
               \\             \\ /
                '------ Base::Class
                       /  \\ \\ \\
                      '    \\ \\ \\
                      |     \\ \\ \\
                      .      \\ \\ '--- The::Latest
                     /|       \\ \\      \\
 With::Some::fantasy  '        \\ \\      '---- The::Latest::Greatest
                     /|         \\ \\
         More::Stuff  '          \\ '- I::Am::Running::Out::Of::Ideas
                     /|           \\
         More::Stuff  '            \\
                     /              '--- Last::One
         More::Stuff

  Safety
    ^
    |                       *Rust
    |           *Java
    | *Python
    |                        *C++
    +-----------------------------> Control





  TODO:

      \\
       .--


          ^
       \\|/     \\/
      <-+->    /\\
       /|\\

        |      |    |      |
        +--  --+    +--  --+   +--  --+
                    |      |   |      |

                     |    |  |     |
             .- -.   .-  -.  ._   _.
             |   |

        .-   -.  .-.
        '-   -'  | |  | |
                      '-'

      \\      |    /  |
       .     '   '   .
       |    /    |    \\

       \\
       /

       /
       \\


       /      \\
      '--    --'
     /          \\

       /   \\
    --'     '--
     /       \\

                       \\         /
       --.--  --.--   --.--   --.--
        /        \\


        |   |
        .   .
       /|   |\\

        |
        .
       / \\

       \\|/
        .
       /|\\


       \\|/
      --.--
       /|\\

       \\|/
      --+--
       /|\\

        |/  \\|
        .    .
        |    |


       -.  -.
       /     \\

        .-  .-
       /     \\


       /   /     \\    \\
      .-  '_     _'   -'


       .-.
      (   )
       '-'

       ..
      (  )
       ''


       .------.
      (        )
       '------'

        ________
       /       /
      /       /
     /_______/


        ________
        \\       \\
         \\       \\
          \\_______\\

       ________
      |________|


       ________
      |        |
      |________|

      .-.
      '-'

        ________
        \\_______\\

       /\\
      /  \\
     /____\\

       /\\
      /  \\
     /    \\
    '------'

       ___
      /   \\
      \\___/

      ______
     /      \\
    /        \\
    \\        /
     \\______/


   .----------------.
   | FFFFFFFFFFFFF  |
   |             .--+
   |             | /
   .-------------+/

        +---------+
        |         |                        +--------------+
        |   NFS   +--+                     |              |
        |         |  |                 +-->|   CacheFS    |
        +---------+  |   +----------+  |   |  /dev/hda5   |
                     |   |          |  |   +--------------+
        +---------+  +-->|          |  |
        |         |      |          |--+
        |   AFS   |----->| FS-Cache |
        |         |      |          |--+
        +---------+  +-->|          |  |
                     |   |          |  |   +--------------+
        +---------+  |   +----------+  |   |              |
        |         |  |                 +-->|  CacheFiles  |
        |  ISOFS  |--+                     |  /var/cache  |
        |         |                        +--------------+
        +---------+

    Svgbob is a diagramming model
    which uses a set of typing characters
    to approximate the intended shape.

           .---.
          /-o-/--
       .-/ / /->
      ( *  \\/
       '-.  \\
          \\ /
           '
    It uses a combination of characters
    which are readily available on your keyboards.

    What can it do?

    #-->
    <--#
                                |/
                              --#--
                               /|
     o->  Basic shapes
        .- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -.
        |                                                    .            :
            +------+   .------.    .------.      /\\        .' `.          :
        |   |      |   |      |   (        )    /  \\     .'     `.        :
            +------+   '------'    '------'    '----'     `.   .'       # :
        |     _______            ________               #   `.'      ^ /  :
        !    /       \\      /\\   \\       \\     o---->   | ^         / /   :
        !   /         \\    /  \\   )       )    <----#   | |  ^ :   / v    :
        !   \\         /    \\  /  /_______/              v |  ! :          :
        !    \\_______/      \\/             o.             o  ! V          :
        !                                    `.~~~~.                      :
        !                                           '.         O          :
        !    .-----------.       .   <.      .>  .    '.     ^  \\         :
        !   (             )     (      )    (     )    :      \\  \\        :
        !    '-----+ ,---'       `>   '      `  <'     '.~~~~> \\  v       :
        !         \\|/                                           *         :
        !          '                            _        __       ___     :
        !    _   __   .-.   .--.   .--.--.    .' '.    ,'  `.   ,'   `.   :
        !   (_) (__) ( 3 ) ( 4  ) ( 4( )4 )  (  6  )  (  7   ) /   8   \\  :
        !             '-'   `--'   `--'--'    `._.'    `.__,'  \\       /  :
        !    .--.        ____          ____           _____     `.___,'   !
        !   ( 5  )     ,'    `.      .'    `.       ,'     `.             !
        !    `--'     /   9    \\    /        \\     /         \\            !
        !             \\        /   (    10    )   (     11    )           !
        !              `.____,'     \\        /     \\         /            !
        !        ______              `.____.'       `._____,'             !
        !      ,'      `.                                                 !
        !     /          \\    .-----. .----.                              !
        !    |     12     |    \\   /   \\    \\                             !
        !    |            |     \\ /     \\    \\                            !
        !     \\          /       '       '----'                           !
        !      `.______,'                                                 !
        `~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'


     o->  Quick logo scribbles
            .---.                      _
           /-o-/--       .--.         |-|               .--.
        .-/ / /->       /--. \\     .--)-|    .--.-.    //.-.\\
       ( *  \\/         / O  )|     |  |-|    |->| |   (+(-*-))
        '-.  \\        /\\ |-//      .  * |    '--'-'    \\\\'-'/
           \\ /        \\ \\|//        \\__/                '--'
            '          '--'

           .----.               _
           |    |           ,--(_)
         __|____|__       _/ .-. \\
        |  ______--|     (_)(   ) )
        `-/.::::.\\-'       \\ `-'_/
         `--------'         `--(_)


     o->  Even unicode box drawing characters are supported
                ┌─┬┐  ╔═╦╗  ╓─╥╖  ╒═╤╕
                ├─┼┤  ╠═╬╣  ╟─╫╢  ╞═╪╡
                └─┴┘  ╚═╩╝  ╙─╨╜  ╘═╧╛
                ╭─┬╮
                ├─┼┤
                ╰─┴╯

     o-> Grids



        .----.        .----.
       /      \\      /      \\            .-----+-----+-----.
      +        +----+        +----.      |     |     |     |          .-----+-----+-----+-----+
       \\      /      \\      /      \\     |     |     |     |         /     /     /     /     /
        +----+        +----+        +    +-----+-----+-----+        +-----+-----+-----+-----+
       /      \\      /      \\      /     |     |     |     |       /     /     /     /     /
      +        +----+        +----+      |     |     |     |      +-----+-----+-----+-----+
       \\      /      \\      /      \\     +-----+-----+-----+     /     /     /     /     /
        '----+        +----+        +    |     |     |     |    +-----+-----+-----+-----+
              \\      /      \\      /     |     |     |     |   /     /     /     /     /
               '----'        '----'      '-----+-----+-----'  '-----+-----+-----+-----+


.---.   .---.
|   +---+   |
+---+   +---+
|   +---+   |
+---+   +---+
|   +---+   |
'---'   '---'



           ___     ___      .---+---+---+---+---.     .---+---+---+---.  .---.   .---.
       ___/   \\___/   \\     |   |   |   |   |   |    / \\ / \\ / \\ / \\ /   |   +---+   |
      /   \\___/   \\___/     +---+---+---+---+---+   +---+---+---+---+    +---+   +---+
      \\___/   \\___/   \\     |   |   |   |   |   |    \\ / \\ / \\ / \\ / \\   |   +---+   |
      /   \\___/   \\___/     +---+---+---+---+---+     +---+---+---+---+  +---+   +---+
      \\___/   \\___/   \\     |   |   |   |   |   |    / \\ / \\ / \\ / \\ /   |   +---+   |
          \\___/   \\___/     '---+---+---+---+---'   '---+---+---+---'    '---'   '---'


     O-> Graphics Diagram

                                                                                 .
        0       3                          P *              Eye /         ^     /
         *-------*      +y                    \\                +)          \\   /  Reflection
      1 /|    2 /|       ^                     \\                \\           \\ v
       *-+-----* |       |                v0    \\       v3           --------*--------
       | |4    | |7      | ◄╮               *----\\-----*
       | *-----|-*     ⤹ +-----> +x        /      v X   \\          .-.<--------        o
       |/      |/       / ⤴               /        o     \\        ( / ) Refraction    / \\
       *-------*       v                 /                \\        '-'               /   \\
      5       6      +z              v1 *------------------* v2    |                o-----o
                                                                   v



     o->  Sequence Diagrams

                                       .--->  F
              A       B      C  D     /
              *-------*-----*---*----*----->  E
                       \\            ^ \\
                        v          /   '--->  G
                         B --> C -'


                              ,-.
                              `-'
                              /|\\
             ,---.             |
             |Bob|            / \\
             `-+-'           Alice
               |    hello      |
               |-------------->|
               |               |
               |  Is it ok?    |
               |<- - - - - - - |
             ,-+-.           Alice
             |Bob|            ,-.
             `---'            `-'
                              /|\\
                               |
                              / \\


                  .─.
                 ( 0 )
                  `-'
                /     \\
               /       \\
              V         V
             .─.         .─.
            ( 1 )       ( 4 )
             `-'         `-' .
           /   \\         |  \\ `.
          /     \\        |   \\  `.
         V       V       |    \\   `.
        .─.      .─.     V     V    V
       ( 2 )    ( 3 )    .─.   .─.   .─.
        `─'      `─'    ( 5 ) ( 6 ) ( 7 )
                         `─'   `─'   `─'

     o-> Plot diagrams

            ▲
        Uin ┊   .------------------------
            ┊   |
            ┊   |
            *---'┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄▶

         Udc▲
    "Udc_OK"┊      .---------------------
            ┊     /  :
            ┊    /   :
            *---'┄┄┄┄:┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄▶
                     :<----->:
            ▲          500ms :
            ┊                :
    Cpu.Qon ┊┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄┄.-----------
            ┊                |  Inactive
            ┊    Active      |
            *----------------'┄┄┄┄┄┄┄┄┄┄┄▶

     o-> Railroad diagrams

                   ┌------┐   .-.  ┌---┐
            o--╮---| elem |--( ; )-| n |--╭--o
               |   └------┘   `-'  └---┘  |
               | ╭------>------╮          |
               | |    ┌---┐    |          |
               ╰-╯-╭--| x |--╮-╰----------╯
               |   |  └---┘  |            |
               |   |   .-.   |            |
               |   `--( , )--'            ^
               |       `-'                |
               |  ╭-------->---------╮    |
               |  |   ┌---┐   .-.    |    |
               ╰--╰-╭-| x |--( , )-╮-╯----╯
                    | └---┘   `-'  |
                    `-------<------'
                                           .------------>---------------.
               ┌-------------┐  .-.   .-.  |  ┌------┐  .-.   ┌-----┐   |    .-.   ┌------┐
          O____| struct_name |_( : )_( | )_|__| name |_( : )__| tpe |___◟___( | )__| body |______O
            ◝  └-------------┘  `-'   `-'   ◜ └------┘  `-'   └-----┘  ◝     `-'   └------┘  ◜
            |                               |                    .-.   |                     |
            |                               `------------<------( , )--'                     |
            |                                                    `-'                         |
            `--------------------------------------------------------------------------------'

     o-> Statistical charts


        E +-------------------------*--+     E |                         o
        D |-------------------*--*--|--*     D |                   o  o  |  o
        C |-------------*--*  |  |  |  |     C |             o  o  |  |  |  |
        B |-------*--*  |  |  |  |  |  |     B |       o  o  |  |  |  |  |  |
        A +-*--*--+--+--+--+--+--+--+--+     A +-o--o--|--|--|--|--|--|--|--|
            5 10 15 20 25 30 35 40 45 50         5 10 15 20 25 30 35 40 45 50



      85.67 ┤                                       ╭╮
      78.20 ┤                                       ││                  ╭╮
      70.73 ┤                                       ││  ╭╮ ╭╮ ╭╮   ╭╮  ╭╯╰─╮
      63.27 ┤                        ╭╮         ╭─╮ ││ ╭╯╰╮│╰─╯╰╮╭╮│╰──╯   │╭
      55.80 ┤   ╭╮                 ╭╮││╭╮ ╭╮╭╮  │ ╰─╯╰─╯  ││    ││││       ╰╯
      48.33 ┤   │╰╮      ╭──╮      │││││╰╮│╰╯│  │         ╰╯    ╰╯╰╯
      40.87 ┤╭╮ │ ╰╮╭╮  ╭╯  ╰─╮╭╮╭─╯╰╯╰╯ ╰╯  ╰──╯
      33.40 ┤││ │  ╰╯╰╮╭╯     ││╰╯
      25.93 ┤││╭╯     ╰╯      ╰╯
      18.47 ┼╯││
      11.00 ┤ ╰╯
            └───────────┴───────────┴───────────┴───────────┴───────────┴────
          2011        2012        2013        2014        2015        2016







     o->  Flow charts
                          .--.            .---.  .---. .---.  .---.    .---.  .---.
                          |  |   OS API   '---'  '---' '---'  '---'    '---'  '---'
                          v  |              |      |     |      |        |      |
                 .-. .-. .-. |              v      v     |      v        |      v
             .-->'-' '-' '-' |            .------------. | .-----------. |  .-----.
             |     \\  |  /   |            | Filesystem | | | Scheduler | |  | MMU |
             |      v . v    |            '------------' | '-----------' |  '-----'
             |_______/ \\_____|                   |       |      |        |
                     \\ /
                      .                        v       |      |        v
                      |     ____              .----.     |      |    .---------.
                      '--> /___/              | IO |<----'      |    | Network |
                                              '----'            |    '---------'
                                                 |              |         |
                                                 v              v         v
                                          .---------------------------------------.
                                          |                  HAL                  |
                                          '---------------------------------------'


                 .---.  .---. .---.  .---.    .---.  .---.
        OS API   '---'  '---' '---'  '---'    '---'  '---'
                   |      |     |      |        |      |
                   v      v     |      v        |      v
                 .------------. | .-----------. |  .-----.
                 |  文
    件
    系
    统
      | | |   调
    度
    器
      | |  | MMU |
                 '------------' | '-----------' |  '-----'
                        |       |      |        |
                        v       |      |        v
                     .----.     |      |    .---------.
                     | IO |<----'      |    |   网
    络
      |
                     '----'            |    '---------'
                        |              |         |
                        v              v         v
                 .---------------------------------------.
                 |              硬
    件
    抽
    象
    层
                   |
                 '---------------------------------------'


     o->  Block diagrams


          vncviewer         .-,(  ),-.
           __  _         .-(          )-.           gateway           vncserver
          [__]|=|  ---->(    internet    )-------> __________ ------> ____   __
         /⠶⠶ /|_|        '-(          ).-'        [_...__...°]       |    | |==|
                             '-.( ).-'                               |____| |  |
                                                                     /⠶⠶⠶ / |__|


                                  Valveless --------.
                                Pulsejet engine    /
                                                  V
                                   _________.------------------+
                        .---------'                           /  -------->
                       /  .-------._________                  \\   thrust-->
                      (  (      _________   `-----------o------+  -------->
                       \\  `----'         '----'         |
                        `------._  __^___.----.         |
                                 ||  |                  |
                       fuel  __^ ||  | ^__spark         |GND
                      intake     ||  |    plug          |
                                 ||  |                  |
                                 ||  |                  |
                             ____||  `------------.     |
                            / .---'               |     |
                            | |                   |     |              +-+-+-+-+-+
                        .---| |---.   __          |     |              |-+-+-+-+-|
                     ___|  +-+-+--|--o  `---------*-----|--------------O-+-+-+-+-|
           .-------> ___  ||||||| |  power        |     *--------------O-+-+-+-+-|
            \\           | ||||||| |  switch       |     |              |-+-+-+-+-|
              Water     `-+-+-+-+-'            +--o-----o--+           +-+-+-+-+-+
              intake       HHO                 |           |
                          Generator            |  +     -  |            Solar panel
                                               +-----------+
                                                  Battery

                   =======
                    =====  symbolic antenna
                     ===
                      =
                      |
                      |          micro henry
                      |          coil w/tuning lug
                      |    .----.
                      |   (.-') |
                      |   (.-') |
                      |   (.-') |      pico farad cap
                      |   (.-'  |    ___  (trimmable)
                      |   |     |   |___|
            PC   ->  .----'-----'---'---'
            Board    `-------------------
                     ground plane (foil)




     o-> Mindmaps

                                                .-->  Alpha
                                               /
                                              .---->  Initial Release
              Planning *-------.             /         \\
                                \\           /           '---> Patch 1
          Initial research       \\         /             \\
                    *             \\       /               '-->  Patch 2
                     \\             \\     .---------> Beta
                      \\             \\   /
                       \\             o o                      _______
                        \\          .---. *--.___             /       \\
                         '------> (     )       '------O->  . Release .
                                   `---' o                   \\_______/
                                   o  o o \\
                                  /    \\ \\ \\
                              .--'      \\ \\ \\
                             /           \\ \\ '----+->  Push backs
                            .             \\ \\      \\
                           /|              \\ \\      '----> Setbacks
                          / .               \\ \\
                         V /|                \\ '-----> Reception
                    Team  / .                 \\
                         v /|                  \\
                 Worklaod / .                   '-->> Career change
                         V /
                     PTO  /
                         V
                     Bug


     o->  It can do complex stuff such as circuit diagrams


           +10-15V           ___0,047R
          *---------o-----o-|___|-o--o---------o----o-------.
        + |         |     |       |  |         |    |       |
        -===-      _|_    |       | .+.        |    |       |
        -===-      .-.    |       | | | 2k2    |    |       |
        -===-    470| +   |       | | |        |    |      _|_
        - |       uF|     '--.    | '+'       .+.   |      \\ / LED
          +---------o        |6   |7 |8    1k | |   |      -+-
                 ___|___   .-+----+--+--.     | |   |       |
                  -═══-    |            |     '+'   |       |
                    -      |            |1     |  |/  BC    |
                   GND     |            +------o--+   547   |
                           |            |      |  |`>       |
                           |            |     ,+.   |       |
                   .-------+            | 220R| |   o----||-+  IRF9Z34
                   |       |            |     | |   |    |+->
                   |       |  MC34063   |     `+'   |    ||-+
                   |       |            |      |    |       |  BYV29     -12V6
                   |       |            |      '----'       o--|<-o----o--X OUT
     6000 micro  - | +     |            |2                  |     |    |
     Farad, 40V ___|_____  |            |--o                C|    |    |
     Capacitor  ~ ~ ~ ~ ~  |            | GND         30uH  C|    |   --- 470
                   |       |            |3      1nF         C|    |   ###  uF
                   |       |            |-------||--.       |     |    | +
                   |       '-----+----+-'           |      GND    |   GND
                   |            5|   4|             |             |
                   |             |    '-------------o-------------o
                   |             |                           ___  |
                   `-------------*------/\\/\\/------------o--|___|-'
                                         2k              |       1k0
                                                        .+.
                                                        | | 5k6 + 3k3
                                                        | | in Serie
                                                        '+'
                                                         |
                                                        GND


      .-------------.
     / Advantages: /
    '-------------'
     -* Plain text format
        Ultimately portable, backward compatible and future proof.
     -* Degrades gracefully
        Even when not using a graphical renderer, it would still looks good
        as text based diagrams. Paste the text in your source code.
     -* Easiest to use. Anyone knows how to edit text.

    """
